@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ConfigDefs.pl 4484 2008-06-17 19:45:00Z sysjkf $
+#   $Id: ConfigDefs.pl 4608 2008-12-11 13:51:32Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -105,6 +105,7 @@ envfromheader			= EnvelopeFromHeader
 envtoheader			= EnvelopeToHeader
 findphishing			= FindPhishingFraud
 fprotd6port			= FpscandPort
+getipfromheader			= ReadIPAddressFromReceivedHeader
 gsscanner			= UseCustomSpamScanner
 gstimeout			= CustomSpamScannerTimeout
 gstimeoutlen			= CustomSpamScannertimeouthistory
@@ -261,6 +262,7 @@ clamavspam		0	no	0	yes	1
 debug			0	no	0	yes	1
 debugspamassassin	0	no	0	yes	1
 deliverinbackground	1	no	0	yes	1
+getipfromheader		0	no	0	yes	1
 lognonspam		0	no	0	yes	1
 logsaactions		0	no	0	yes	1
 logsilentviruses	0	no	0	yes	1
@@ -280,7 +282,7 @@ usedefaultswithmanyrecips	0	no	0	yes	1
 # These should be checked for dir existence
 [Simple,Dir]
 incomingworkdir		/var/spool/MailScanner/incoming
-lockfiledir		/tmp
+lockfiledir		/var/spool/MailScanner/incoming/Locks
 
 # Check the first word of these for file existence
 [Simple,File]
@@ -295,6 +297,7 @@ VirusScannerDefinitions	/etc/MailScanner/virus.scanners.conf
 
 # Check these to ensure they are just numbers
 [Simple,Number]
+AntiwordTimeout			50
 BayesRebuild			0
 Children			5
 clamavmaxreclevel               8
@@ -389,6 +392,7 @@ WorkPerms		0600
 #
 
 [First,YesNo]
+AddTextOfDoc		0	no	0	yes	1
 AllowExternal		0	no	0	yes	1
 AllowPartial		0	no	0	yes	1
 ArchivePublicKeys	0	no	0	yes	1
@@ -511,6 +515,7 @@ NormalRBLs			1
 ReqSpamAssassinScore		6
 
 [First,Other]
+Antiword			/usr/bin/antiword -f
 AttachmentCharset		ISO-8859-1
 attachimageinternalname
 attachimagename
