@@ -246,13 +246,14 @@ sub LFileName {
 #  }
 
 sub ReadQf {
-  my($this, $message) = @_;
+  my($this, $message, $getipfromheader) = @_;
 
   my($RQf) = $message->{store}{inhhandle};
 
   my %metadata;
   my($InHeader, $InSubject, $InDel, @headers, $msginfo, $from, @to, $subject);
-  my($ip, $sender, @acl, @aclc, @aclm, $line, $acltype);
+  my($ip, $sender, %acl, %aclc, %aclm, $line, $acltype);
+  my($read1strcvd, $ipfromheader);
 
   #print STDERR "ReadQf for " . $message->{id} . "\n";
 
@@ -294,16 +295,16 @@ sub ReadQf {
     $acltype = $1;
     if($acltype =~ /^acl[cm]?$/) {
       # we need to handle acl vars differently
-      if($line =~ /^(\w+) (\d+)$/) {
+      if($line =~ /^(\w+|_[[:alnum:]_]+) (\d+)$/) {
         my $buf;
         my $pos = $1;
         my $len = $2;
         if ($acltype eq "acl") {
-          $acl[$pos]->[0] = [];
+          $acl{$pos}->[0] = [];
         } elsif ($acltype eq "aclc") {
-          $aclc[$pos]->[0] = [];
+          $aclc{$pos}->[0] = [];
         } elsif ($acltype eq "aclm") {
-          $aclm[$pos]->[0] = [];
+          $aclm{$pos}->[0] = [];
         } else {
           # invalid format
           last;
@@ -316,11 +317,11 @@ sub ReadQf {
           last;
         }
         if ($acltype eq "acl") {
-          $acl[$pos]->[0] = $buf;
+          $acl{$pos}->[0] = $buf;
         } elsif ($acltype eq "aclc") {
-          $aclc[$pos]->[0] = $buf;
+          $aclc{$pos}->[0] = $buf;
         } elsif ($acltype eq "aclm") {
-          $aclm[$pos]->[0] = $buf;
+          $aclm{$pos}->[0] = $buf;
         } else {
           # invalid format
           last;
@@ -337,9 +338,9 @@ sub ReadQf {
     }
     next;
   }
-  $metadata{aclvars} = \@acl;
-  $metadata{aclcvars} = \@aclc;
-  $metadata{aclmvars} = \@aclm;
+  $metadata{aclvars} = \%acl;
+  $metadata{aclcvars} = \%aclc;
+  $metadata{aclmvars} = \%aclm;
 
   # If it was an invalid queue file, log a warning and tell caller
   unless (defined $line) {
@@ -579,6 +580,20 @@ sub ReadQf {
 	  chomp($message->{subject} = $2);
 	  $InSubject = 1;
 	}
+        if ("received:" eq lc $1) {
+          my $received = $2;
+          if ($received =~ /\[(\d+\.\d+\.\d+\.\d+)\]/i) {
+            unless ($read1strcvd) {
+              $ipfromheader = $1;
+              $read1strcvd = 1;
+            }
+          } elsif ($received =~ /\[([\dabcdef.:]+)\]/i) {
+            unless ($read1strcvd) {
+              $ipfromheader = $1;
+              $read1strcvd = 1;
+            }
+          }
+        }
       }
       # Track anything we may be in the middle of
       $InDel = ($InDel && $InHeader);
@@ -593,6 +608,10 @@ sub ReadQf {
 		 "- continuing anyway.");
   }
 
+  # If we were told to read the IP from the header and it was there...
+  $message->{clientip} = $ipfromheader
+    if $getipfromheader && $read1strcvd && $ipfromheader ne "";
+
   # We should have the last header built but not pushed
   # onto the metadata headers array at this point...
   exists $header->{name} and push @{$metadata{headers}},$header;
@@ -1127,24 +1146,25 @@ sub CreateQf {
 
   # ACLs patch starts here
   # Add the separate ACL Vars
-  my @acl  = @{$metadata->{aclvars}};
-  my @aclc = @{$metadata->{aclcvars}};
-  my @aclm = @{$metadata->{aclmvars}};
-  my $greatestacl = $#acl;
-  $greatestacl    = $#aclc if $#aclc > $greatestacl;
-  $greatestacl    = $#aclm if $#aclm > $greatestacl;
-  for($i=0; $i<=$greatestacl; $i++) {
-    if($acl[$i]) {
-      $Qfile .= "-acl " . $i . " " . length($acl[$i]->[0]) . "\n";
-      $Qfile .= $acl[$i]->[0] . "\n";
+  my %acl  = %{$metadata->{aclvars}};
+  my %aclc = %{$metadata->{aclcvars}};
+  my %aclm = %{$metadata->{aclmvars}};
+  foreach(keys %acl){
+    if($acl{$_}) {
+      $Qfile .= "-acl " . $_ . " " . length($acl{$_}->[0]) . "\n";
+      $Qfile .= $acl{$_}->[0] . "\n";
     }
-    if($aclc[$i]) {
-      $Qfile .= "-aclc " . $i . " " . length($aclc[$i]->[0]) . "\n";
-      $Qfile .= $aclc[$i]->[0] . "\n";
+  }
+  foreach(keys %aclc){
+    if($aclc{$_}) {
+      $Qfile .= "-aclc " . $_ . " " . length($aclc{$_}->[0]) . "\n";
+      $Qfile .= $aclc{$_}->[0] . "\n";
     }
-    if($aclm[$i]) {
-      $Qfile .= "-aclm " . $i . " " . length($aclm[$i]->[0]) . "\n";
-      $Qfile .= $aclm[$i]->[0] . "\n";
+  }
+  foreach(keys %aclm){
+    if($aclm{$_}) {
+      $Qfile .= "-aclm " . $_ . " " . length($aclm{$_}->[0]) . "\n";
+      $Qfile .= $aclm{$_}->[0] . "\n";
     }
   }
 
@@ -1388,7 +1408,7 @@ sub SendMessageEntity {
 
 sub CreateBatch {
   my $this = shift;
-  my($batch) = @_;
+  my($batch, $onlyid) = @_;
 
   my($queuedirname, $queuedir, $MsgsInQueue, $mtime);
   my($DirtyMsgs, $DirtyBytes, $CleanMsgs, $CleanBytes);
@@ -1397,6 +1417,7 @@ sub CreateBatch {
   my(%ModDate, $mta, $file, $tmpdate, $invalidfiles);
   my(@SortedFiles, $id, $newmessage, @queuedirnames);
   my($batchempty, $CriticalQueueSize, $headerfileumask);
+  my($getipfromheader);
 
   # Old code left over from single queue dir
   #$queuedirname = $global::MS->{inq}{dir};
@@ -1416,6 +1437,7 @@ sub CreateBatch {
   # If there are too many messages in the queue, start processing in
   # directory storage order instead of date order.
   $CriticalQueueSize = MailScanner::Config::Value('criticalqueuesize');
+  $getipfromheader = MailScanner::Config::Value('getipfromheader');
 
   # Set what we will need the umask to be
   $headerfileumask = $global::MS->{work}->{fileumask};
@@ -1524,11 +1546,14 @@ sub CreateBatch {
       ($queuedirname, $file) = ($1,$2) if $file =~ /^(.*)\/([^\/]+)$/;
       next unless $file =~ /$mta->{HFileRegexp}/;
       $id = $1;
+      # If they want a particular message id, ignore it if it doesn't match
+      next if $onlyid ne "" && $id ne $onlyid;
 
       #print STDERR "Adding $id to batch\n";
 
       # Lock and read the qf file. Skip this message if the lock fails.
-      $newmessage = MailScanner::Message->new($id, $queuedirname);
+      $newmessage = MailScanner::Message->new($id, $queuedirname,
+                                              $getipfromheader);
       if ($newmessage eq 'INVALID') {
         $invalidfiles .= "$id ";
         next;
