@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SA.pm 4183 2007-09-24 15:03:52Z sysjkf $
+#   $Id: SA.pm 4645 2009-01-08 10:34:46Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ use Compress::Zlib;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4183 $, 10;
+$VERSION = substr q$Revision: 4645 $, 10;
 
 # Attributes are
 #
@@ -69,6 +69,22 @@ my $HighSpamCacheLife = 3*60*60;  # Lifetime of high spam from last seen
 my $VirusesCacheLife  = 48*60*60; # Lifetime of viruses from last seen
 my $ExpireFrequency   = 10*60;    # How often to run the expiry of the cache
 
+sub CreateTempDir {
+  my($runasuser,$satmpdir) = @_;
+
+  # Create the $TMPDIR for SpamAssassin if necessary, then check we can
+  # write to it. If not, change to /tmp.
+  lstat $satmpdir;
+  unless (-d _) {
+    unlink $satmpdir;
+    mkdir $satmpdir or warn "Could not create SpamAssassin temporary directory $satmpdir, $!";
+  }
+  chmod 0700, $satmpdir unless $satmpdir =~ /^\/tmp/;
+  chown $runasuser, -1, $satmpdir;
+
+  $ENV{'TMPDIR'} = $satmpdir;
+}
+
 sub initialise {
   my($RebuildBayes, $WantLintOnly) = @_; # Start by rebuilding the Bayes database?
 
@@ -78,6 +94,35 @@ sub initialise {
   @SAsuccessqueue = ();
   $SAsuccessqsum  = 0;
 
+  # If the "debug-sa" command-line flag was given, then we want to print out
+  # the current time at the start of each line that is sent to STDERR,
+  # in particular the stuff from SpamAssassin.
+  if ($MailScanner::SA::Debug) {
+    my $result;
+    # Do a trial run of awk to see if it is going to work on this system.
+    eval {
+      $result = `echo 'Hello,World' | awk '{printf \"%s %s\\n\", strftime(\"%T\"), \$0}' 2>&1`;
+      #print "Result is \"$result\"\n";
+    };
+    #print "Eval result = \"$@\"\n";
+
+    # If the trial worked...
+    if ($result =~ /\d+:\d.*Hello,World/) {
+      #print STDERR "It Succeeded!\n";
+      #select STDERR; $| = 1;
+      # Re-open STDERR with the current time stuck on the front of each line.
+      # It should work okay as we have just tried it out.
+      open STDERR, "| awk '{printf \"%s %s\\n\", strftime(\"%T\"), \$0}'";
+      select STDOUT;
+    } else {
+      print STDERR "\n\n*****\n";
+      print STDERR "If 'awk' (with support for the function strftime) was\n";
+      print STDERR "available on your \$PATH then all the SpamAssassin debug\n";
+      print STDERR "output would have the current time added to the start of\n";
+      print STDERR "every line, making debugging far easier.\n*****\n\n";
+    }
+  }
+
   # Can't just do this when sendmail.pl loads, as we are still running as
   # root then & spamassassin will get confused when we are later running
   # as something else.
@@ -91,17 +136,6 @@ sub initialise {
   # So, set ENV{HOME} to desired directory, or undef it to force it to get home
   # using getpwnam of $> (EUID)
 
-  # Create the $TMPDIR for SpamAssassin if necessary, then check we can
-  # write to it. If not, change to /tmp.
-  my $tmpdir = MailScanner::Config::Value('spamassassintempdir');
-  mkdir $tmpdir;
-  stat $tmpdir; # Is the directory writeable?
-  $tmpdir = '/tmp' unless -d _ && -r _ && -w _ && -x _;
-  $ENV{'TMPDIR'} = $tmpdir;
-  MailScanner::Log::InfoLog("SpamAssassin temporary working directory is %s",
-                            $tmpdir);
-  print STDERR "SpamAssassin temp dir = $tmpdir\n";
-
   unless (MailScanner::Config::IsSimpleValue('usespamassassin') &&
           !MailScanner::Config::Value('usespamassassin')) {
     $settings{dont_copy_prefs} = 1; # Removes need for home directory
@@ -229,6 +263,14 @@ sub initialise {
                                   "%s to signal starting",
                                   $MailScanner::SA::BayesRebuildStartLock);
       }
+      # 20090107 Get exclusive lock on the startlock
+      #flock($RebuildStartH, $LOCK_EX);
+      flock($RebuildStartH, $LOCK_EX);
+      $RebuildStartH->seek(0,0);
+      $RebuildStartH->autoflush();
+      print $RebuildStartH "1\n";
+      flock($RebuildStartH, $LOCK_UN);
+      $RebuildStartH->close();
 
       # Get an exclusive lock on the bayes rebuild lock file
       my $RebuildLockH = new FileHandle;
@@ -256,15 +298,24 @@ sub initialise {
                                   "failed with error: %s", $@)
           if $@;
 
+        # Unlock the "starting" lock
+        $RebuildStartH = new FileHandle;
+        $RebuildStartH->open("+>$MailScanner::SA::BayesRebuildStartLock");
+        flock($RebuildStartH, $LOCK_EX);
+        $RebuildStartH->seek(0,0);
+        $RebuildStartH->autoflush();
+        print $RebuildStartH "0\n";
+        flock($RebuildStartH, $LOCK_UN);
+        $RebuildStartH->close();
         # Unlock the bayes rebuild lock file
-        unlink($MailScanner::SA::BayesRebuildLock);
+        #20090107 unlink($MailScanner::SA::BayesRebuildLock);
         flock($RebuildLockH, $LOCK_UN);
         $RebuildLockH->close();
         MailScanner::Log::InfoLog('SpamAssassin Bayes database rebuild completed');
       }
 
       # Now the rebuild has properly finished, we let the other children back
-      unlink $MailScanner::SA::BayesRebuildStartLock;
+      #20090107 unlink $MailScanner::SA::BayesRebuildStartLock;
       $RebuildStartH->close();
     }
 
@@ -399,7 +450,16 @@ sub Checks {
     # Shared locks are handed out even when someone else is trying to
     # get an exclusive lock, so long as at least 1 other shared lock
     # already exists.
-    if (-e $MailScanner::SA::BayesRebuildStartLock) {
+    #20090107 if (-e $MailScanner::SA::BayesRebuildStartLock) {
+      my $fh = new FileHandle;
+      $fh->open("+<" . $MailScanner::SA::BayesRebuildStartLock);
+      flock($fh, $LOCK_EX);
+      $fh->seek(0,0);
+      my $line = <$fh>;
+      flock($fh, $LOCK_UN);
+      $fh->close();
+
+      if ($line =~ /1/) {
       # Do we wait for Bayes rebuild to occur?
       if ($MailScanner::SA::WaitForRebuild) {
         $0 = 'MailScanner: waiting for Bayes rebuild';
@@ -407,20 +467,37 @@ sub Checks {
         # This must not take more than 1 hour or we are in trouble!
         #MailScanner::Log::WarnLog("Waiting for rebuild start request to disappear");
         my $waiter = 0;
-        for ($waiter = 0; $waiter<3600 &&
-             -e $MailScanner::SA::BayesRebuildStartLock; $waiter+=10) {
-          sleep 10;
+        for ($waiter = 0; $waiter<3600 && $line =~ /1/; $waiter+=30) {
+             #-e $MailScanner::SA::BayesRebuildStartLock; $waiter+=10) {
+          sleep 30;
+          $fh = new FileHandle;
+          $fh->open("+<" . $MailScanner::SA::BayesRebuildStartLock);
+          flock($fh, $LOCK_EX);
+          $fh->seek(0,0);
+          $line = <$fh>;
+          flock($fh, $LOCK_UN);
+          $fh->close();
           #MailScanner::Log::WarnLog("Waiting for start request to disappear");
         }
         # Did it take too long?
-        unlink $MailScanner::SA::BayesRebuildStartLock if $waiter>=3590;
+        #unlink $MailScanner::SA::BayesRebuildStartLock if $waiter>=3590;
+        if ($waiter>=4000) {
+          $fh = new FileHandle;
+          $fh->open("+>" . $MailScanner::SA::BayesRebuildStartLock);
+          flock($fh, $LOCK_EX);
+          $fh->seek(0,0);
+          $fh->autoflush();
+          print $fh "0\n";
+          flock($fh, $LOCK_UN);
+          $fh->close();
+        }
         #MailScanner::Log::WarnLog("Start request has disappeared");
         $0 = 'MailScanner: checking with SpamAssassin';
       } else {
         # Return saying we are skipping SpamAssassin this time
         return (0,0, 'SpamAssassin rebuilding', 0);
       }
-    }
+      }
 
     $Lockopen = 0;
     $RebuildLockH = new FileHandle;
@@ -599,7 +676,11 @@ sub Checks {
       #             "SAHitList = $SAHitList\n";
 
       # Write the record to the cache *****
-      CacheResult($md5digest, $SAResult, $HighScoring, compress($SAHitList), $SAScore, compress($SAReport));
+      # Don't cache "timed out" results.
+      if ($SAHitList ne MailScanner::Config::LanguageValue($message, 'satimedout')) {
+        CacheResult($md5digest, $SAResult, $HighScoring,
+                    compress($SAHitList), $SAScore, compress($SAReport));
+      }
 
 
       # Unlock and close the lockfile
@@ -853,8 +934,14 @@ sub SAForkAndTest {
     $SAReport  = <$pipe>;
     #print STDERR "Read SAHitList = $SAHitList " . scalar(localtime) . "\n";
     # Not sure if next 2 lines should be this way round...
-    waitpid $pid, 0;
     $pipe->close();
+    waitpid $pid, 0;
+    ## JKF 4.71.7 waitpid $pid, 0;
+    #my $kid;
+    #do {
+    #  $kid = waitpid(-1,&POSIX::WNOHANG);
+    #} until $kid == -1;
+    ## JKF 4.71.7 end
     $PipeReturn = $?;
     alarm 0;
     $pid = 0;
@@ -911,7 +998,7 @@ sub SAForkAndTest {
       $SAsuccessqsum += (shift @SAsuccessqueue)?1:-1
         if @SAsuccessqueue>$queuelength;
       #print STDERR "Failure: sum = $SAsuccessqsum\n";
-      $SAsuccessqsum = 0 if $SAsuccessqsum<0;
+      $SAsuccessqsum = 1 if $SAsuccessqsum<1;
 
       if ($SAsuccessqsum>$maxfailures && @SAsuccessqueue>=$queuelength) {
         MailScanner::Log::WarnLog("SpamAssassin timed out (with no network" .
