@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ZMailer.pm 4372 2008-04-02 15:45:26Z sysjkf $
+#   $Id: ZMailer.pm 4522 2008-08-20 15:19:23Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use Encode;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4372 $, 10;
+$VERSION = substr q$Revision: 4522 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -231,7 +231,7 @@ sub TFileName {
 # Change for V4: returns lower-case $from and @to
      # REVISO LEOH
 sub ReadQf {
-  my($this, $message) = @_;
+  my($this, $message, $getipfromheader) = @_;
 
   my($RQf) = $message->{store}{inhdhandle};
 
@@ -239,6 +239,7 @@ sub ReadQf {
   my($ip, $TOline, $RcvdFromLine);
   my($Line, $Flags);
   my($TOFound, $FROMFound, $IPFound);
+  my($read1strcvd, $ipfromheader);
 
   #$message->{store}->print();
 
@@ -328,6 +329,17 @@ sub ReadQf {
       if ($Line =~ /^Subject:\s*(\S.*)?$/i) {
         $message->{subject} = $1;
       }
+      if ($Line =~ /^Received: .+\[(\d+\.\d+\.\d+\.\d+)\]/i) {
+        unless ($read1strcvd) {
+          $ipfromheader = $1;
+          $read1strcvd = 1;
+        }
+      } elsif ($Line =~ /^Received: .+\[([\dabcdef.:]+)\]/i) {
+        unless ($read1strcvd) {
+          $ipfromheader = $1;
+          $read1strcvd = 1;
+        }
+      }
       # JKF Testing code, remove before production release
       #if ($Line =~ /^X-ClientIP:/i) {
       #  $message->{clientip} = $1 if $Line =~ /(\d+\.\d+\.\d+\.\d+)/;
@@ -337,6 +349,10 @@ sub ReadQf {
     }
   }
 
+  # If we were told to get the IP address from the header, and it was there...
+  $message->{clientip} = $ipfromheader
+    if $getipfromheader && $read1strcvd && $ipfromheader ne "";
+
   # Decode ISO subject lines into UTF8
   # Needed for UTF8 support in MailWatch 2.0
   eval {
@@ -765,6 +781,7 @@ sub CreateBatch {
   my(%ModDate, $mta, $file, $tmpdate, $invalidfiles);
   my(@SortedFiles, $id, $newmessage, @queuedirnames);
   my($batchempty, $CriticalQueueSize, $headerfileumask);
+  my($getipfromheader);
 
   # Old code left over from single queue dir
   #$queuedirname = $global::MS->{inq}{dir};
@@ -783,6 +800,7 @@ sub CreateBatch {
   # If there are too many messages in the queue, start processing in
   # directory storage order instead of date order.
   $CriticalQueueSize = MailScanner::Config::Value('criticalqueuesize');
+  $getipfromheader = MailScanner::Config::Value('getipfromheader');
 
   # Set what we will need the umask to be
   $headerfileumask = $global::MS->{work}->{fileumask};
@@ -891,7 +909,8 @@ sub CreateBatch {
       #print STDERR "Adding $id to batch\n";
 
       # Lock and read the qf file. Skip this message if the lock fails.
-      $newmessage = MailScanner::Message->new($id, $queuedirname);
+      $newmessage = MailScanner::Message->new($id, $queuedirname,
+                                              $getipfromheader);
       if ($newmessage eq 'INVALID') {
         $invalidfiles .= "$id ";
         next;
